/*******************************************************************************
 * Copyright (c) 2000, 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *     Frank Benoit <benoit@tionex.de>
 *******************************************************************************/
module dwt.graphics.TextStyle;

import dwt.DWT;
import dwt.graphics.Font;
import dwt.graphics.Color;
import dwt.graphics.GlyphMetrics;

import tango.util.Convert;
/**
 * <code>TextStyle</code> defines a set of styles that can be applied
 * to a range of text.
 * <p>
 * The hashCode() method in this class uses the values of the public
 * fields to compute the hash value. When storing instances of the
 * class in hashed collections, do not modify these fields after the
 * object has been inserted.
 * </p>
 * <p>
 * Application code does <em>not</em> need to explicitly release the
 * resources managed by each instance when those instances are no longer
 * required, and thus no <code>dispose()</code> method is provided.
 * </p>
 *
 * @see TextLayout
 * @see Font
 * @see Color
 *
 * @since 3.0
 */
public class TextStyle {

    /**
     * the font of the style
     */
    public Font font;

    /**
     * the foreground of the style
     */
    public Color foreground;

    /**
     * the background of the style
     */
    public Color background;

    /**
     * the underline flag of the style
     *
     * @since 3.1
     */
    public bool underline;

    /**
     * the strikeout flag of the style
     *
     * @since 3.1
     */
    public bool strikeout;

    /**
     * the GlyphMetrics of the style
     *
     * @since 3.2
     */
    public GlyphMetrics metrics;

    /**
     * the baseline rise of the style.
     *
     * @since 3.2
     */
    public int rise;

/**
 * Create a new text style with the specified font, foreground
 * and background.
 *
 * @param font the font of the style, <code>null</code> if none
 * @param foreground the foreground color of the style, <code>null</code> if none
 * @param background the background color of the style, <code>null</code> if none
 */
public this (Font font, Color foreground, Color background) {
    if (font !is null && font.isDisposed()) DWT.error (DWT.ERROR_INVALID_ARGUMENT);
    if (foreground !is null && foreground.isDisposed()) DWT.error (DWT.ERROR_INVALID_ARGUMENT);
    if (background !is null && background.isDisposed()) DWT.error (DWT.ERROR_INVALID_ARGUMENT);
    this.font = font;
    this.foreground = foreground;
    this.background = background;
}

/**
 * Compares the argument to the receiver, and returns true
 * if they represent the <em>same</em> object using a class
 * specific comparison.
 *
 * @param object the object to compare with this object
 * @return <code>true</code> if the object is the same as this object and <code>false</code> otherwise
 *
 * @see #hashCode()
 */
public override int opEquals(Object object) {
    if (object is this) return true;
    if (object is null) return false;
    if (!(cast(TextStyle)object)) return false;
    TextStyle style = cast(TextStyle)object;
    if (foreground !is null) {
        if ( foreground != style.foreground ) return false;
    } else if (style.foreground !is null) return false;
    if (background !is null) {
        if ( background != style.background ) return false;
    } else if (style.background !is null) return false;
    if (font !is null) {
        if (font != style.font) return false;
    } else if (style.font !is null) return false;
    if (metrics !is null || style.metrics !is null) return false;
    if (underline !is style.underline) return false;
    if (strikeout !is style.strikeout) return false;
    if (rise !is style.rise) return false;
    return true;
}

/**
 * Returns an integer hash code for the receiver. Any two
 * objects that return <code>true</code> when passed to
 * <code>equals</code> must return the same value for this
 * method.
 *
 * @return the receiver's hash
 *
 * @see #equals(Object)
 */
public override hash_t toHash() {
    int hash = 0;
    if (foreground !is null) hash ^= foreground.toHash();
    if (background !is null) hash ^= background.toHash();
    if (font !is null) hash ^= font.toHash();
    if (metrics !is null) hash ^= metrics.toHash();
    if (underline) hash ^= hash;
    if (strikeout) hash ^= hash;
    hash ^= rise;
    return hash;
}

/**
 * Returns a string containing a concise, human-readable
 * description of the receiver.
 *
 * @return a string representation of the <code>TextStyle</code>
 */
public char[] toString () {
    char[] buffer = "TextStyle {";
    int startLength = buffer.length;
    if (font != null) {
        if (buffer.length > startLength) buffer ~= ", ";
        buffer ~= "font=";
        buffer ~= font.toString;
    }
    if (foreground != null) {
        if (buffer.length > startLength) buffer ~= ", ";
        buffer ~= "foreground=";
        buffer ~= foreground.toString;
    }
    if (background != null) {
        if (buffer.length > startLength) buffer ~= ", ";
        buffer ~= "background=";
        buffer ~= background.toString;
    }
    if (underline) {
        if (buffer.length > startLength) buffer ~= ", ";
        buffer ~= "underlined";
    }
    if (strikeout) {
        if (buffer.length > startLength) buffer ~= ", ";
        buffer ~= "striked out";
    }
    if (rise != 0) {
        if (buffer.length > startLength) buffer ~= ", ";
        buffer ~= "rise=";
        buffer ~= to!(char[])(rise);
    }
    if (metrics != null) {
        if (buffer.length > startLength) buffer ~= ", ";
        buffer ~= "metrics=";
        buffer ~= metrics.toString;
    }
    buffer ~= "}";
    return buffer;
}

}
